@@ -169,7 +169,8 @@ usage()
   echo "    --data-directory=<directory>            | default: ${JS7_AGENT_DATA}"
   echo "    --config-directory=<directory>          | default: ${JS7_AGENT_CONFIG_DIR}"
   echo "    --sigkill-delay=<seconds>               | send SIGTERM and delayed SIGKILL signal, default: ${JS7_AGENT_SIGKILL_DELAY}"
-  echo "    --timeout=<seconds>                     | timeout for terminating jobs on Agent stop"
+  echo "    --timeout=<seconds>                     | timeout for cancelling the Agent on stop"
+#  echo "    --stop-timeout=<seconds>                | timeout for terminating jobs on Agent stop"
   echo "    --java-options=<java options>           | default: ${JS7_AGENT_JAVA_OPTIONS}; see https://kb.sos-berlin.com/x/uYo7B"
   echo "    --watchdog=<path>                       | default: ${JS7_AGENT_WATCHDOG_DEFAULT}; see https://kb.sos-berlin.com/x/MbpfCQ"
   echo "  switches:"
@@ -262,7 +263,7 @@ start_agent_in_container()
   #remove_folder "${JS7_AGENT_DATA}/work"
   #create_folder "${JS7_AGENT_DATA}/work"
   
-  test -z "${JS7_AGENT_KILL_SCRIPT}" || echo "--kill-script option is not longer supported"
+  test -z "${JS7_AGENT_KILL_SCRIPT}" || echo "--kill-script option is no longer supported"
   test -z "${JS7_AGENT_HTTPS_PORT}" || HTTPS_PORT_OPTION="--https-port=${JS7_AGENT_HTTPS_PORT}"
 
   echo "\"${JAVABIN}\" ${JS7_AGENT_JAVA_OPTIONS} -classpath \"${JS7_AGENT_CLASSPATH}\" js7.agent.main.AgentMain --http-port=\"${JS7_AGENT_HTTP_PORT}\" ${HTTPS_PORT_OPTION} --config-directory=\"${JS7_AGENT_CONFIG_DIR}\" --data-directory=\"${JS7_AGENT_DATA}\" --job-working-directory=\"${JS7_AGENT_WORK_DIR}\""
@@ -273,7 +274,7 @@ start_agent_in_container()
   then
     create_pid_file $!
     JS7_AGENT_PID=$!
-    echo "...JS7 Agent(${JS7_AGENT_PORT}) is started with pid=${JS7_AGENT_PID}!"
+    echo "...JS7 Agent(${JS7_AGENT_PORT}) is starting with pid=${JS7_AGENT_PID}!"
     wait "${JS7_AGENT_PID}" || JS7_AGENT_EXIT=$?
     restart_container "${JS7_AGENT_PID}"
   fi
@@ -281,7 +282,7 @@ start_agent_in_container()
 
 restart_container()
 {
-  echo "...JS7 Agent terminates with exit code ${JS7_AGENT_EXIT}"
+  echo "...JS7 Agent terminated with exit code ${JS7_AGENT_EXIT}"
   
   call_crash_pid_file_killer "$1"
   
@@ -314,16 +315,22 @@ shutdown_agent()
   if running
   then
   	create_sigkill_delay_file
-  	echo "...try to terminate the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
+  	echo "...trying to terminate the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
   	if [ "$1" = "SIGKILL" ]
     then
       #  called by trap in start-docker if container sends INT or TERM
       http_command POST "{\"TYPE\": \"ShutDown\", \"processSignal\": \"SIGKILL\"}"
-    elif [ "${JS7_AGENT_STOP_TIMEOUT}" -gt "-2" ]
+    elif [ "${JS7_AGENT_CANCEL_TIMEOUT}" -gt "-2" ]
     then
-      # waits for running jobs
-      http_command POST "{\"TYPE\": \"ShutDown\"}"
+      # waits for running jobs except --stop-timeout=0
+      if [ -n "${JS7_AGENT_STOP_TIMEOUT}" ] && [ "${JS7_AGENT_STOP_TIMEOUT}" -eq "0" ]
+      then
+      	http_command POST "{\"TYPE\": \"ShutDown\", \"processSignal\": \"SIGTERM\"}"
+      else
+        http_command POST "{\"TYPE\": \"ShutDown\"}"
+      fi
     else
+      # without --timeout
       http_command POST "{\"TYPE\": \"ShutDown\", \"processSignal\": \"SIGTERM\"}"
     fi
   else
@@ -342,16 +349,26 @@ shutdown_agent()
       then
         printf "...waiting for termination"
       fi
-      if [ "${JS7_AGENT_STOP_TIMEOUT}" -gt "-1" ] && [ "${loop_counter}" -ge "${JS7_AGENT_STOP_TIMEOUT}" ]
+      if [ -n "${JS7_AGENT_STOP_TIMEOUT}" ] && [ "${loop_counter}" -ge "${JS7_AGENT_STOP_TIMEOUT}" ]
       then
-      	echo "..timeout reached"
-      	echo "...try to cancel (SIGKILL) the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
-      	kill_err=0
-      	no_mercy "${JS7_AGENT_PID}" || kill_err=$?
-      	if [ "$kill_err" = "0" ]
-      	then
-      	  exit 0
-      	fi
+        # post shutdown once again with SIGTERM
+        JS7_AGENT_STOP_TIMEOUT=
+        if running
+        then
+          http_command POST "{\"TYPE\": \"ShutDown\", \"processSignal\": \"SIGTERM\"}"
+        fi 
+      fi
+      # --timeout != never
+      if [ "${JS7_AGENT_CANCEL_TIMEOUT}" -gt "-1" ] && [ "${loop_counter}" -ge "${JS7_AGENT_CANCEL_TIMEOUT}" ]
+      then
+        echo "..timeout reached"
+        echo "...trying to cancel (SIGKILL) the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
+        kill_err=0
+        no_mercy "${JS7_AGENT_PID}" || kill_err=$?
+        if [ "$kill_err" = "0" ]
+        then
+          exit 0
+        fi
       fi
       printf ".."
       loop_counter=$(( loop_counter + 2 ))
@@ -367,7 +384,7 @@ abort_agent()
   if running
   then
   	create_sigkill_delay_file
-    echo "...try to terminate forcibly the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
+    echo "...trying to forcibly terminate the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
     http_command POST "{\"TYPE\": \"EmergencyStop\"}" 2>/dev/null
   else
     echo "...not started!"
@@ -375,7 +392,7 @@ abort_agent()
   sleep 2
   if running
   then
-    echo "...could not abort the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}!"
+    echo "...could not cancel the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}!"
     exit "${JS7_AGENT_EXIT}"
   else
     JS7_AGENT_EXIT=0
@@ -407,7 +424,7 @@ cancel_agent()
     #if ps -p "${JS7_AGENT_PID}" > /dev/null 2>&1
     #then
     #  echo "...try once more with SIGKILL."
-      echo "...try to cancel (SIGKILL) the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
+      echo "...trying to cancel (SIGKILL) the JS7 Agent(${JS7_AGENT_PORT}) with pid=${JS7_AGENT_PID}."
       no_mercy "${JS7_AGENT_PID}"
     #else
     #  echo "...canceled!"
@@ -428,16 +445,16 @@ normalize_agent_classpath_user_var()
 
 no_mercy()
 {
-  JS7_AGENT_STOP_TIMEOUT=-2
+  JS7_AGENT_CANCEL_TIMEOUT=-2
   kill_err=0
   kill -9 "$1" || kill_err=$?
   if [ "$kill_err" = "0" ]
   then
-    echo "...canceled!"
+    echo "...cancelled!"
     remove_pid_file
     exit 0
   else
-    echo "...could not cancel the JS7 Agent(${JS7_AGENT_PORT}) with pid=$1!"
+    echo "...could not cancel (SIGKILL) the JS7 Agent(${JS7_AGENT_PORT}) with pid=$1!"
     exit 3
   fi
 }
@@ -446,7 +463,7 @@ switch_over()
 {
   if running
   then
-    echo "...try to switch over the Agent cluster of JS7 Agent(${JS7_AGENT_PORT})."
+    echo "...trying to switch-over the Agent cluster of JS7 Agent(${JS7_AGENT_PORT})."
     http_command POST "{\"TYPE\": \"ClusterSwitchOver\"}"
   else
     echo "...not started!"
@@ -474,7 +491,7 @@ command_with_agent_client()
 {
   JS7_AGENT_EXIT=0
   JS7_AGENT_LIBDIR="${JS7_AGENT_HOME}/lib"
-  "${JAVABIN}" -Xmx100m -classpath "${JS7_AGENT_LIBDIR}/http-client:${JS7_AGENT_LIBDIR}/sos/*:${JS7_AGENT_LIBDIR}/3rd-party/*" js7.agent.client.main.AgentClientMain --config-directory="${JS7_AGENT_CONFIG_DIR}" --data-directory="${JS7_AGENT_DATA}" "$(get_url_cutted)" "$1" || JS7_AGENT_EXIT=$?
+  "${JAVABIN}" -Xmx100m -classpath "${JS7_AGENT_LIBDIR}/http-client:${JS7_AGENT_LIBDIR}/sos/*:${JS7_AGENT_LIBDIR}/3rd-party/*" js7.agent.client.main.SubagentClientMain --config-directory="${JS7_AGENT_CONFIG_DIR}" --data-directory="${JS7_AGENT_DATA}" "$1" || JS7_AGENT_EXIT=$?
 }
 
 command_with_curl()
@@ -663,11 +680,6 @@ get_url()
   fi
 }
 
-get_url_cutted()
-{
-  echo "$(get_url | cut -d / -f 1,2,3)"
-}
-
 add_ip_address()
 {
   IP_ADDRESS=$(echo "$1" | grep ":" | cut -d : -f 1)
@@ -731,7 +743,7 @@ is_integer()
   	test "$1" -eq "$1" 2>/dev/null
     if [ $? -ne 0 ]
     then
-      echo "$1 is not an integer"
+      # echo "$1 is not an integer"
       return 1
     fi
   else
@@ -772,6 +784,7 @@ end()
   unset JS7_AGENT_PID
   unset JS7_AGENT_LIBDIR
   unset JS7_AGENT_CLIENT_CLASSPATH
+  unset JS7_AGENT_CANCEL_TIMEOUT
   unset JS7_AGENT_STOP_TIMEOUT
   unset JS7_AGENT_SIGKILL_DELAY
   unset JS7_AGENT_SIGKILL_DELAY_DEFAULT
@@ -844,7 +857,8 @@ JS7_AGENT_PORT=
 WITH_CURL=1
 WITH_FAILOVER=1
 JS7_WITHOUT_WATCHDOG=1
-JS7_AGENT_STOP_TIMEOUT=-2
+JS7_AGENT_CANCEL_TIMEOUT=
+JS7_AGENT_STOP_TIMEOUT=
 for param in "$@"
 do
   case "$param" in
@@ -858,7 +872,9 @@ do
                                ;;
      --sigkill-delay=*)        JS7_AGENT_SIGKILL_DELAY=$(echo "$param" | sed 's/--sigkill-delay=//' | sed 's/^"//' | sed 's/"$//')
                                ;;
-     --timeout=*)              JS7_AGENT_STOP_TIMEOUT=$(echo "$param" | sed 's/--timeout=//' | sed 's/^"//' | sed 's/"$//')
+     --stop-timeout=*)         JS7_AGENT_STOP_TIMEOUT=$(echo "$param" | sed 's/--stop-timeout=//' | sed 's/^"//' | sed 's/"$//')
+                               ;;
+     --timeout=*)              JS7_AGENT_CANCEL_TIMEOUT=$(echo "$param" | sed 's/--timeout=//' | sed 's/^"//' | sed 's/"$//')
                                ;;
      --kill-script=*)          JS7_AGENT_KILL_SCRIPT=$(echo "$param" | sed 's/--kill-script=//' | sed 's/^"//' | sed 's/"$//')
                                ;;
@@ -881,23 +897,51 @@ JS7_AGENT_HTTP_PORT=$(echo "${JS7_AGENT_HTTP_PORT}" | sed 's/^://')
 JS7_AGENT_HTTPS_PORT=$(echo "${JS7_AGENT_HTTPS_PORT}" | sed 's/^://')
 test -z "${JS7_AGENT_HTTP_PORT}" && JS7_AGENT_HTTP_PORT=$(add_ip_address "${JS7_AGENT_DEFAULT_HTTP_PORT}" "")
 JS7_AGENT_PORT=$(get_port "${JS7_AGENT_HTTP_PORT}")
-if [ "never" = "${JS7_AGENT_STOP_TIMEOUT}" ]
+
+if [ -n "${JS7_AGENT_STOP_TIMEOUT}" ]
 then
-  JS7_AGENT_STOP_TIMEOUT=-1
+  if [ -z "${JS7_AGENT_CANCEL_TIMEOUT}" ]
+  then
+  	echo "--stop-timeout requires --timeout"
+  	more_info
+    exit 2
+  fi
+  if ! is_integer "${JS7_AGENT_STOP_TIMEOUT}" || [ "${JS7_AGENT_STOP_TIMEOUT}" -lt "0" ]
+  then
+    echo "--stop-timeout has to be a non-negative integer"
+    more_info
+    exit 2
+  fi
 fi
-if ! is_integer "${JS7_AGENT_STOP_TIMEOUT}"
+	
+if [ -n "${JS7_AGENT_CANCEL_TIMEOUT}" ]
 then
-  JS7_AGENT_STOP_TIMEOUT=-2
-fi
-if [ "${JS7_AGENT_STOP_TIMEOUT}" -lt "-1" ]
-then
-  JS7_AGENT_STOP_TIMEOUT=-2
+  if [ "never" = "${JS7_AGENT_CANCEL_TIMEOUT}" ]
+  then
+  	JS7_AGENT_CANCEL_TIMEOUT=-1
+  else
+    if ! is_integer "${JS7_AGENT_CANCEL_TIMEOUT}" || [ "${JS7_AGENT_CANCEL_TIMEOUT}" -lt "0" ]
+    then
+      echo "--timeout has to be a non-negative integer or 'never'"
+      more_info
+      exit 2
+    fi
+    if [ -n "${JS7_AGENT_STOP_TIMEOUT}" ] && [ "${JS7_AGENT_STOP_TIMEOUT}" -ge "${JS7_AGENT_CANCEL_TIMEOUT}" ]
+    then
+      echo "--stop-timeout has to be lower than --timeout"
+      more_info
+      exit 2
+    fi
+  fi
+else
+  JS7_AGENT_CANCEL_TIMEOUT=-2
 fi
+
 if [ -n "${JS7_AGENT_SIGKILL_DELAY}" ]
 then
   if ! is_integer "${JS7_AGENT_SIGKILL_DELAY}" || [ "${JS7_AGENT_SIGKILL_DELAY}" -lt "0" ]
   then
-    echo "--sigkill-delay has to be a positive integer. ${JS7_AGENT_SIGKILL_DELAY_DEFAULT}seconds are used"
+    echo "--sigkill-delay has to be a non-negative integer. ${JS7_AGENT_SIGKILL_DELAY_DEFAULT}seconds are used"
     JS7_AGENT_SIGKILL_DELAY=
   fi
 fi
@@ -963,6 +1007,7 @@ test -z "${JS7_AGENT_JAVA_OPTIONS}" && JS7_AGENT_JAVA_OPTIONS="${JAVA_OPTIONS}"
 echo "${JS7_AGENT_JAVA_OPTIONS}" | grep -e "-Xms" >/dev/null 2>/dev/null || JS7_AGENT_JAVA_OPTIONS="${JS7_AGENT_JAVA_OPTIONS} -Xms100m"
 echo "${JS7_AGENT_JAVA_OPTIONS}" | grep -e "-Dfile.encoding" >/dev/null 2>/dev/null || JS7_AGENT_JAVA_OPTIONS="${JS7_AGENT_JAVA_OPTIONS} -Dfile.encoding=UTF-8"
 echo "${JS7_AGENT_JAVA_OPTIONS}" | grep -e "HeapDumpOnOutOfMemoryError" >/dev/null 2>/dev/null || JS7_AGENT_JAVA_OPTIONS="${JS7_AGENT_JAVA_OPTIONS} -XX:+HeapDumpOnOutOfMemoryError"
+echo "${JS7_AGENT_JAVA_OPTIONS}" | grep -e "ExitOnOutOfMemoryError" >/dev/null 2>/dev/null || JS7_AGENT_JAVA_OPTIONS="${JS7_AGENT_JAVA_OPTIONS} -XX:+ExitOnOutOfMemoryError"
 
 
 ##########################
